
# q-expansion of newform 3888.2.a.r, downloaded from the LMFDB on 21 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3888
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [2], [0], [-2], [1], [-2], [-5], [-2], [0], [-5], [-6], [12], [5], [-12], [-10], [-14], [7], [1], [2], [-11], [-1], [2], [12], [5], [12], [-1], [12], [-11], [12], [-13], [2], [0], [-12], [10], [13], [7], [-12], [-12], [12], [0], [-18], [24], [-25], [12], [-19], [-19], [11], [12], [25], [2], [26], [-6], [-22], [-2], [-14], [-14], [13], [1], [-14], [13], [-24], [12], [-12], [6], [0], [13], [-6], [24], [-6], [-26], [-12], [23], [11], [16], [36], [-22], [-37], [-24], [-30], [24], [26], [-22], [7], [-24], [-10], [-10], [-31], [-22], [-35], [-36], [14], [-7], [-12], [36], [24], [0], [-14], [29], [30], [-29], [38], [34], [14], [-12], [35], [34], [-34], [12], [-5], [4], [11], [12], [17], [20], [14], [-12], [-22], [0], [12], [35], [-41], [2], [46], [-5], [36], [5], [-2], [-48], [41], [31], [-14], [48], [-7], [10], [-1], [-36], [17], [38], [22], [29], [36], [31], [2], [-11], [0], [23], [-12], [-36], [10], [18], [22], [-11], [-22], [-13], [22], [20], [10], [6], [60], [-36], [-60], [1], [-48], [24], [-24], [47], [-26], [29], [14], [-60], [42], [-10], [-26], [-23], [58], [61], [24], [-49], [11], [28], [14], [30], [-24], [-12], [24], [25], [-60], [1], [-48], [-11], [12], [-16], [-34], [-24], [-50], [-65], [13], [-2], [36], [-26], [55], [41], [1], [12], [-36], [-41], [12], [-62], [-40], [-18], [12], [-67], [-46], [-46], [47], [1], [-46], [-48], [-24], [-7], [72], [0], [61], [-48], [-1], [48], [-24], [-48], [70], [-18], [-25], [-41], [26], [36], [50], [53], [-70], [50], [10], [-48], [-11], [17], [-29], [-24], [50], [-25], [0], [-1], [62], [-42], [72], [38], [-23], [-24], [-72], [-19], [-31], [12], [-77], [65], [12], [-59], [-6], [-34], [31], [22], [-38], [-65], [-38], [-50], [-56], [1], [-43], [13], [-53], [36], [35], [43], [34], [22], [5], [70], [47], [7], [-74], [-7], [-12], [-17], [-38], [14], [12], [-34], [-48], [-54], [14], [37], [-84], [-36], [-77], [-37], [22], [1], [70], [-36], [-31], [12], [41], [50], [-13], [-84], [12], [-38], [41], [-74], [-19], [12], [-48], [29], [-12], [29], [31], [-26], [-49], [50], [-6], [-36], [60], [-26], [-72], [37], [48], [-24], [60], [36], [-60], [-77], [24], [41], [72], [55], [-60], [-12], [-48], [26], [34], [6], [71], [0], [70], [12], [-79], [38], [0], [-73], [-50], [-26], [-22], [62], [12], [-73], [24], [-36], [-34], [-67], [78], [12], [-31], [-5], [-82], [-60], [-58], [-46], [-79], [-43], [-72], [24], [-53], [-84], [53], [-48], [58], [72], [48], [23], [-24], [96], [-22], [41], [-48], [-59], [62], [72], [-36], [-12], [61], [48], [36], [1], [86], [-1], [-60], [-7], [-14], [74], [-103], [-83], [60], [-77], [-12], [11], [-22], [2], [-25], [42], [50], [84], [-89], [36], [-70], [94], [47], [-46], [-22], [-41], [10], [-86], [-72], [108], [58]]
